% function [Status] = rs_ADS_IQSink (ComplexIQ, ArbClock, Playback, RFFreq, RFLev, ViRscStr, StrRemotePath, StrWvFile, StrCopyright, StrComment)
% *************************************************************************
% 
% Copyright: (c) 2006 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
% *************************************************************************
% This function takes a 1D I-data and a 1D Q-data array, combines them and
% generates a file equipped with all mandatory and optional tags as
% specified in the manual. This file is transfered to the remote instrument
% where it can be played immediately.
%%
%                                     hard disk of your pc
%%
% @return Status      Status = 0, if file generation or transmission fails
%                     Status = 1, if generation and transmission succeeds
% *************************************************************************

function [Status] = rs_ADS_IQSink (ComplexIQ, ArbClock, Playback, RFFreq, RFLev, ViRscStr, StrRemotePath, StrWvFile, StrCopyright, StrComment)

    Status = 0;
    save_local = 1;
    VisaIntf = 'ni';
    
    % chek number of arguments
    if (nargin ~= 10)
        disp ('*** Wrong number of input arguments.')
        return;
    end

    % check arb clock rate
    if (~isnumeric(ArbClock))
        disp ('*** Clock rate must be numeric.');
        return;
    end

    % check RF frequency
    if (~isnumeric(RFFreq))
        disp ('*** RF Frequency must be numeric.');
        return;
    end

    % check RF level
    if (~isnumeric(RFLev))
        disp ('*** RF Level must be numeric.');
        return;
    end

    % check playback to be numeric
    if (~isnumeric(Playback))
        disp ('*** playback flag must be numeric.');
        return;
    end

    % check visa resource string to be a string
    if (~ischar(ViRscStr))
        disp ('*** VISA resource must be a string');
        return;
    end

    % check comment to be a string
    if (~ischar(StrComment))
        disp ('*** Comment must be a string');
        return;
    end
    
    % check comment to be a string
    if (~ischar(StrCopyright))
        disp ('*** Copyright must be a string');
        return;
    end

    % check path information to be a string
    if (~ischar(StrRemotePath))
        disp ('*** Remote path must be a string');
        return;
    end
    if (StrRemotePath(end) ~= '\')
        StrRemotePath = strcat(StrRemotePath,'\');
    end

    % check filename to be a string and check for correct extension
    if (~ischar(StrWvFile))
        disp ('*** Remote filename must be a string.');
        return
    end
    if (~strcmpi(StrWvFile(end-2:end),'.wv')==1)
        disp ('*** Remote file extension must be .wv.');
        return;
    end

    % set parameters
    IQinfo.path      = StrRemotePath;
    IQinfo.filename  = StrWvFile;             
    IQinfo.comment   = StrCopyright;
    IQinfo.copyright = StrComment;
    IQinfo.I_data    = real(ComplexIQ);
    IQinfo.Q_data    = imag(ComplexIQ);
    IQinfo.clock     = ArbClock;
   
    % check I and Q data for correct length (>1 number)
    if ((length(IQinfo.I_data) ~= 1) && (length(IQinfo.Q_data) ~= 1)) 
        if length(IQinfo.I_data) ~= length(IQinfo.Q_data)
            disp ('*** I and Q data array has to be of the same length.');
            return;
        end
    end

    IQ_data_len = length(IQinfo.I_data);

    % prepare IQ data array, format is IQIQIQ...
    try
        IQ_data = single(zeros(1, 2*IQ_data_len));  
        IQ_data(1:2:end) = single(IQinfo.I_data);
        clear IQinfo.I_data;
        IQ_data(2:2:end) = single(IQinfo.Q_data);
				clear IQinfo.Q_data;
    catch
        disp ('*** Failed to assemble data structure.');
        return;
    end

    % if max value of the envelope is greater than 1.0 
    % all IQ data must be normalized.
    max_IQ_data = max(abs(IQ_data));
    peak = max_IQ_data;
    if max_IQ_data > 1.0
        IQ_data = IQ_data / max_IQ_data;
        % disp(['Maximum envelope value is ',num2str(max_IQ_data,5),', waveform normalized to 1.0 max.']);
    	peak = 1.0;
    else
        max_IQ_data = 1.0;
    end

    % compute peak, rms and crest factor 
    rms  = sqrt(mean(IQ_data(1:2:end).*IQ_data(1:2:end) + IQ_data(2:2:end).*IQ_data(2:2:end))) / max_IQ_data;
    crf  = 20*log10(peak/rms);

    %disp( 'Waveform parameters:' );
    %disp(['  Samples : ',num2str(IQ_data_len)]);
    %disp(['  Clock   : ',num2str(IQinfo.clock)]);
    %disp(['  Peak    : ',num2str(peak,5)]);
    %disp(['  RMS     : ',num2str(rms,5)]);
    %disp(['  Crest F : ',num2str(crf,5)]);

    % range is 16 bits for analog outputs
    % ===================================
    % +1.0 ---> +32767
    %  0.0 --->      0
    % -1.0 ---> -32767
    IQ_data        = floor((IQ_data*32767+0.5)); 
    len            = num2str(length(IQ_data)*2 + 3);

    % open file for write (binary mode)
    WV_file_id = fopen(IQinfo.filename,'wb');
    if (WV_file_id < 0)
        disp ('Failed to open output file.');
        return;
    end
    
    %disp ('Writing output file...');
    
    % header tag must always be the very first one
    % SMU-WV works also for SMJ, SMATE, AMU, AFQ
    fprintf (WV_file_id, '%s','{TYPE: SMU-WV, 0}' ); 

    % comment and copyright tags are optional
    if (~(isequal(IQinfo.comment,'')))
        fprintf (WV_file_id, '%s','{COMMENT: ',IQinfo.comment,'}' );
    end
    if (~(isequal(IQinfo.copyright,'')))
        fprintf (WV_file_id, '%s','{COPYRIGHT: ',IQinfo.copyright,'}' );
    end

    % Level offset tag for RMS and peak in dB related to full scale.
    % The full scale is 1.0 for the envelope (IQ vector length).
    fprintf (WV_file_id, '%s','{LEVEL OFFS: ',num2str(20*log10(1.0/rms)),', ',num2str(20*log10(1.0/peak)),'}' );

    % The time stamp tag is optional but it is a good idea to have it in.
    szDate    = datestr(date,29);
    szTime    = datestr(clock,13);
    date_time = ['{DATE: ' szDate ';' szTime '}'];
    fprintf (WV_file_id, '%s', date_time);

    % The clock rate and samples tag are mandatory.
    fprintf (WV_file_id, '%s','{CLOCK: ', num2str(IQinfo.clock),'}' ); 
    fprintf (WV_file_id, '%s','{SAMPLES: ', num2str(IQ_data_len),'}' );

    % The marker list is optional.
    if (isfield(IQinfo,'markerlist'))

        % Use the data leength also as the length of the marker information. 
        % This ensures that the markers remain in sync with the data.
        % Not mandatory but required to stay synchronized.
        fprintf (WV_file_id, '%s','{CONTROL LENGTH: ',num2str(IQ_data_len),'}');

        if (isfield(IQinfo.markerlist,'one'))
            fprintf (WV_file_id, '%s','{MARKER LIST 1: ',IQinfo.markerlist.one,'}' );
        end
        if (isfield(IQinfo.markerlist,'two'))
            fprintf (WV_file_id, '%s','{MARKER LIST 2: ',IQinfo.markerlist.two,'}' );
        end
        if (isfield(IQinfo.markerlist,'three'))
            fprintf (WV_file_id, '%s','{MARKER LIST 3: ',IQinfo.markerlist.three,'}' );
        end
        if (isfield(IQinfo.markerlist,'four'))
            fprintf (WV_file_id, '%s','{MARKER LIST 4: ',IQinfo.markerlist.four,'}' );
        end
    end

    % write waveform tag
    fprintf (WV_file_id, '%s','{WAVEFORM-',len,': 0,#' );       % 0 = start at address 0
    fwrite (WV_file_id, IQ_data, 'int16');
    fprintf (WV_file_id, '%s','}' ); 

    % close waveform output file
    fclose  (WV_file_id);
    
    % free memory
    clear IQ_data;

    % ------------------------------------------------------------------------
    %
    % Transfer waveform file to instrument
    %
    % ------------------------------------------------------------------------

    %
    % use VISA interface from National Instruments to connect via TCP/IP
    %
    [status, InstrObj] = rs_ADS_connect ('visa', VisaIntf, ViRscStr);
    if (status<1)
        disp (['*** Return status from rs_ADS_connect() is : ' num2str(status)]);
        return;
    end
    
    %disp ('Checking for instrument errors...');
    stat = read_error_queue (InstrObj);
    if (stat < 1)
        disp ('*** Failed to read error queue.');
        return;
    end

    %disp ('Sending waveform file...');

    % open binary file for read
    WV_file_id = fopen(IQinfo.filename, 'rb');
    if (WV_file_id < 0)
        disp ('*** Failed to open local waveform file for read.');
        return;
    end

    % read entire waveform file into memory
    % the fread command returns 2*<filesize> of data
    try
        [rawdata, leng] = fread (WV_file_id, 'uchar');
    catch
        fclose (WV_file_id);
        disp ('*** Failed to read local waveform file to memory.');
        return;
    end

    % close file
    fclose(WV_file_id);

    % keep file on local hd
    if (save_local ~= 1)
        delete (sprintf('%s', IQinfo.filename));
    end

    cLength = strcat(num2str(length(num2str(leng))),num2str(leng));
    samples = sprintf('#%s',cLength);

    % create command for data upload
    command = sprintf(':MMEM:DATA ''%s%s'',%s',IQinfo.path,IQinfo.filename,samples);

    
    % ============================================================= 
      
    % for GPIB connections require EOI
 	if (strcmp(get(InstrObj,'Type'),'gpib')==0 || strcmp(get(InstrObj,'Type'),'visa-gpib')==0)
        [stat, Result] = rs_send_query (InstrObj, ':SYST:COMM:GPIB:LTER EOI; *OPC?');
        if (stat < 1 || Result(1) ~= '1')
            disp ('*** Failed to set instrument LTER mode.');
            return;
        end
    end
        
	% do not assert EOI line after transfer
	EoiStat = InstrObj.EOIMode;
	InstrObj.EOIMode = 'off';
    
    % open instrument connection
    try
        fopen (InstrObj);
    catch
        disp ('*** rs_generate_wave(): Failed to open instrument connection.');
        return;
    end
    
    % start with sending the command part
    try
        fwrite (InstrObj, command, 'uchar');
    catch
        disp (['*** Failed to send command (' command ').']);
        fclose (InstrObj);
        return;
    end
    
    % transfer data block by block 
	Blocks        = 0;
	BytesRemain   = leng;
    pcOutBuf      = 1;                      % pointer in output buffer
    
    while (BytesRemain > 0)
		
        % set bytes to send and limit to the size of the output buffeer
        BytesToSend = BytesRemain;
		if (BytesToSend > InstrObj.OutputBufferSize)
			BytesToSend = InstrObj.OutputBufferSize;
        end
        
        % last package to send 
        if (BytesRemain == BytesToSend)
            % enable EOI again to terminate transfer 
            InstrObj.EOIMode = 'on';
        end
        
        % get output data as portion of entire array
        StartIndex = uint32 (pcOutBuf);
        StopIndex  = uint32 (pcOutBuf+BytesToSend-1);
        Percent    = int16 ((1.0-BytesRemain/leng)*100.0);

        % update the user
        %disp (['Next block : ' num2str(StartIndex) ' - ' num2str(StopIndex) ' / ' num2str(Percent) '%']);
        
        OutDataBuf = rawdata (StartIndex:StopIndex);
        
        % write binary data to instrument 
        try
            fwrite (InstrObj, OutDataBuf, 'uchar');
        catch
            disp (['*** Failed to send binary data (index=' num2str(pcOutBuf) ').']);
            fclose (InstrObj);
            return;
        end
        
        % compute remaining bytes 
        BytesRemain = BytesRemain - BytesToSend;
        pcOutBuf    = pcOutBuf    + BytesToSend;

        Blocks = Blocks + 1;
    end
		
    % set back to old state 
    InstrObj.EOIMode = EoiStat;

    % close connection
    try
        fclose (InstrObj);
    catch
        disp ('*** Failed to close instrument connection.');
        return;
    end

    %disp (['Sent ' num2str(Blocks), ' blocks; ', num2str(BytesRemain), ' bytes remaining.']);

    % for GPIB connections turn back to standard
 	if (strcmp(get(InstrObj,'Type'),'gpib')==0 || strcmp(get(InstrObj,'Type'),'visa-gpib')==0)
        [stat, Result] = rs_send_query (InstrObj, ':SYST:COMM:GPIB:LTER STAN; *OPC?');
        if ((stat < 1) || (Result(1) ~= '1'))
            disp ('*** Failed to set instrument LTER mode.'); 
            return;
        end
    end
   
    stat = read_error_queue (InstrObj);
    if (stat < 1)
        disp ('*** Failed to read error queue.');
        return;
    end

    % =============================================================
     
    % activate playback if required
    stat = 1;
    if (Playback==1)
        stat = play (InstrObj, '1', IQinfo.path, IQinfo.filename, RFFreq, RFLev);
    end
    if (Playback==2)
        stat = play (InstrObj, '2', IQinfo.path, IQinfo.filename, RFFreq, RFLev);
    end
    if (stat < 1)
        disp ('*** Failed to activate ARB.');
        return;
    end
    
    stat = read_error_queue (InstrObj);
    if (stat < 1)
        disp ('*** Instrument errors occured.');
        return;
    end
    
    %disp ('ready.');
    
    Status = 1;
return;

% end of function rs_generate_wave 

%
% function - activate playback
%
function [Status] = play (InstrObj, num, path, filename, RFFreq, RFLev)
    
    Status = 0;
    
    [stat, Result] = rs_send_query (InstrObj, sprintf(':SOUR%s:BB:ARB:WAV:SEL ''%s%s''; *OPC?',num,path,filename));
    if (stat<0 || Result(1)~='1'), return; end
    stat = rs_send_command (InstrObj, sprintf(':SOUR%s:BB:ARB:STAT ON', num));
    if (stat<0), return; end
    stat = rs_send_command (InstrObj, sprintf(':OUTP%s:STAT ON', num));
    if (stat<0), return; end
    [stat, Result] = rs_send_query (InstrObj, sprintf(':FREQ %f Hz; *OPC?', RFFreq));
    if (stat<0 || Result(1)~='1'), return; end
    [stat, Result] = rs_send_query (InstrObj, sprintf(':LEV %f dBm; *OPC?', RFLev));
    if (stat<0 || Result(1)~='1'), return; end
    
    Status = 1;
return;

%
% function - check marker array
%
function valid = check_marker_array(marray)
    
    valid = 0;
    
    if (isnumeric(marray) ~= 1)
        disp('*** Marker list must contain numeric values.');
        return
    end
    if ((size(marray(:,1),1)) ~= (size(marray(:,2),1)))
        disp('*** Marker list must contain PAIRS [Position Value] of values.');
        return
    end
    for i=1:1:size(marray(:,2))
        if (~((marray(i,2) == 1) || (marray(i,2) == 0)))
            disp('*** Marker value must be binary.');
            return
        end
    end
    
    valid = 1;
return;

%
% function - building marker
%
function mstring = build_marker_string(marray)
    mstring = '';
    for i=1:1:size(marray,1)-1
        mstring = sprintf('%s%u:%u;',mstring,marray(i,1),marray(i,2));
    end
    mstring = sprintf('%s%u:%u',mstring,marray(size(marray,1),1),marray(size(marray,1),2));
return;

%
% function - read error queue
%
function [Status] = read_error_queue(InstrObj)
    
    Status = 0;
    Errors = 0;
    answer = 'error';
    
    while answer(1) ~= '0'
        [stat, answer] = rs_send_query (InstrObj, 'SYST:ERR?');
        if (stat < 1)
            disp ('*** Failed to read instrument error queue.');
            return;
        end
        if (answer(1) ~= '0')
            disp (['*** Instrument Error: ' answer]);
            Errors = Errors + 1;
        end
    end

    if (Errors == 0)
        Status = 1;
    end
return;

% end of file
