% *************************************************************************
% 
% Copyright: (c) 2011 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
%                8.3.2011       T. Roeder, Rohde&Schwarz
%
% This script demonstates connecting to an instrument by using the
% Rohde & Schwarz MATLAB Toolkit. The R&S MATLAB Toolkit requires the
% MATLAB Instrument Control Toolbox. In the following section the
% use of the various supported hardware interfaces is explained in
% detail.
%
% VISA Interface
% =====================================================================
%  General rs_connect() call syntax:
%    rs_connect ('visa', '<ni|agilent|tek>', '<visa resource string>');
%  The 'visa' identifier is mandatory and sets the interface type.
%  The second identifier is also mandatory and depends on the hardware
%  used. Details can be found in the MATLAB Instrument Control Toolbox
%  documentation.
%  The 'visa resource string' is mandatory and depends on the hardware
%  interface and the instrument. The following section describes the
%  syntax of this string.
%
%  LAN (VXI-11) interface
%    'TCPIP::<ip-address>::INSTR'     e.g. TCPIP::192.168.0.200::INSTR
%    'TCPIP::<hostname>::INSTR'       e.g. TCPIP::rssmu200a100123::INSTR
%                                          TCPIP::smbv100a255025::INSTR
%
%  GPIB interface
%    'GPIB0::<primary address>::INSTR  GPIB0::28::INSTR'
%
%  USB interface
%    'USB::<manufacturer id>::<device id>::<serial>::0::INSTR'
%
%    <manufacturer id>   =           0x0AAD
%
%    INSTRUMENT          |           <device id>
%    -------------------------------------------
%    R&S SMW200A         |           0x92
%    R&S AFQ100A         |           0x4B
%    R&S AMU200A         |           0x55
%    R&S SMATE200A       |           0x46
%    R&S SMBV100A        |           0x5F
%
%    A detailed list of device ids can be found in the R&S MATLAB Toolkit
%    documentation.
%
% GPIB Interface:
% =====================================================================
%  General rs_connect() call syntax:
%    rs_connect ('gpib', 'advantech|agilent|cec|contec|ics|iotech|
%                       keithley|mcc|ni>, <board number>, 
%                       <primary address> [, <secondary address>]);
%  The 'gpib' identifier is mandatory and sets the interface type.
%  The second identifier is also mandatory and depends on the used
%  hardware.
%  The board number is usually zero for the first installed interface.
%  The primary address must match the address setting of the instrument.
%  Most R&S generators use 28 as the default address.
%
% TCP/IP Socket Connections:
% =====================================================================
%  General rs_connect() call syntax:
%    rs_connect ('tcpip', '<hostname>');
%  The 'tcpip' identifier is mandatory and sets the connection type to a
%  raw TCPIP socket connection.                       
%  The hostname is the network name or the IP address of the
%  target instrument, e.g. 'rssmu200a100123' or '192.168.0.200'
%
% *************************************************************************

disp( '  ' );
disp( '=== CONNECTION EXAMPLE ===' );
disp( '  ' );

%   Use the VISA interface from National Instruments and
%   connect via GPIB board number 0 to an instrument with address 28
% [status, InstrObject] = rs_connect ('visa', 'ni', 'GPIB0::28::INSTR');

%   Use the VISA interface from National Instruments and
%   connect via LAN (VXI-11)
  [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::rssmu200a101485::INSTR' );
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::smbv100a255025::INSTR' );

%   Use the raw TCP/IP socket connection on port 5025
% [status, InstrObject] = rs_connect( 'tcpip', 'smbv100a255025' );

if( ~status )
    disp( ['*** Return status from rs_connect() is : ' num2str(status)] );
    return;
end

while(1)
    
    % The default timeout in the MATLAB Instrument Control Toolbox is 10
    % seconds. We reduce this to 1 second.
    InstrObject.Timeout = 1;

    % Query the instrument ID by using the rs_send_query() function
    [status, Result] = rs_send_query( InstrObject, '*IDN?' );
    if( ~status ), break, end;

    disp( ['*IDN? response : ' Result] );

    % Query the instrument options by using the rs_send_query() function
    [status, Result] = rs_send_query( InstrObject, '*OPT?' );
    if( ~status ), break, end;
    
    disp( ['*OPT? response : ' Result] );

    % Query the instrument error queue by using the rs_send_query() function
    [status, Result] = rs_send_query( InstrObject, 'SYST:ERR?' );
    if( ~status ), break, end;
    
    disp( ['SYST:ERR? response : ' Result] );

    % Send instrument reset and do not wait for completion
    disp( 'Resetting Instrument...' );
    [status] = rs_send_command( InstrObject, '*RST' );
    if( ~status ), break, end;

    break;
end

% delete instrument object
delete( InstrObject );

% clean up variables
clear;

return;



