% *************************************************************************
% 
% Copyright: (c) 2011 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
%                16.3.2011       T. Roeder, Rohde&Schwarz
%
% This program demonstrates using the MATLAB toolkit for the
% purpose of generating and uploading a QAM modulated signal to the 
% vector signal generator.
%
% For instrument connection details please see Connect.m 
%
% *************************************************************************

% check if the required toolbox are available
if( ~license( 'checkout', 'communication_toolbox' ) )
    disp( 'Statistics Toolbox is not available.' );
    return;
end
if( ~license( 'checkout', 'signal_toolbox' ) )
    disp( 'Signal Processing Toolbox is not available.' );
    return;
end

% Use the VISA interface from National Instruments and
% connect via GPIB board number 0 to an instrument with address 28
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'GPIB0::28::INSTR' );

% Use the VISA interface from National Instruments and
% connect via LAN (VXI-11)
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::rssmu200a101485::INSTR' );
  [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::smbv100a255005::INSTR' );

% Use the raw TCP/IP socket connection on port 5025
% [status, InstrObject] = rs_connect( 'tcpip', 'smbv100a255025' );

if( ~status )
    clear;
    disp( 'rs_connect() failed.' );
    return;
end

% target instrument path for waveform file
% InstrTargetPath = 'D:\';                  % MS Windows based, e.g. SMU, SMJ, SMATE, AFQ
% InstrTargetPath = '/var/smbv/';           % Linux based, flash memory, e.g. SMBV
  InstrTargetPath = '/hdd/';                % Linux based, hard drive, e.g. SMBV 
% InstrTargetPath = '/var/user/';           % Linux based, SMW200A

StartARB         = 1;                       % start in path A
KeepLocalFile    = 0;                       % waveform only temporarily saved
LocalFileName    = 'qam.wv';                % The local and remote file name

% *************************************************************************
% Create Signal
% *************************************************************************
    
% set the waveform parameters
Fsample       = 10e6;                       % sample frequecy 10 MHz
Ttotal        = 100e-6;                     % total play time 100 us

Tsample = 1/Fsample;                        % resulting sample time
Points  = round( Ttotal / Tsample );        % resulting number of waveform points
k = 0:1:Points-1;                           % point count

QAM_type = 16;

IN_data =  randi( [0 QAM_type-1], Points, 1 );  

% this requires the communication toolbox
QAM_data = qammod( IN_data, QAM_type );     % create QAM 16

% create raised cosine filter
% this requires the signal processing toolbox
F_response = firrcos( 32, 0.4, 0, 1.0, 'normal' );

% low pass filter the signal
QAM_filtered = filter( F_response, 1, QAM_data );

I_data = real( QAM_filtered );
Q_data = imag( QAM_filtered );

% *************************************************************************
% Setup Waveform Structure
% *************************************************************************

IQInfo.I_data         = I_data;
IQInfo.Q_data         = Q_data;
IQInfo.comment        = 'QAM';              % optional comment
IQInfo.copyright      = 'Rohde&Schwarz';    % optional copyright
IQInfo.clock          = Fsample;            % sample rate
IQInfo.no_scaling     = 0;                  % scale level
IQInfo.path           = InstrTargetPath;    % location on instrument 
IQInfo.filename       = LocalFileName;      % local and remote file name
IQInfo.markerlist.one = [[0 1];[1 0]];      % marker signal 1
IQInfo.markerlist.two = [[0 1];[Points/2 0]];

% *************************************************************************
% Plot Data
% *************************************************************************

rs_visualize( Fsample, IQInfo.I_data, IQInfo.Q_data );

% *************************************************************************
% Instrument Setup
% *************************************************************************

% check for R&S device, we also need the *IDN? result later...
disp( 'Checking instrument...' );
[status, InstrIDN] = rs_send_query( InstrObject, '*IDN?' );
if( ~status ); clear;  return; end
if( isempty( strfind( InstrIDN, 'Rohde&Schwarz' ) ) )
    disp('This is not a Rohde&Schwarz device.');
    clear; return;
end

% reset the instrument
[status, OPCResponse] = rs_send_query( InstrObject, '*RST; *OPC?' );
if( ~status ); clear;  return; end
[status] = rs_send_command( InstrObject, '*CLS' );
if( ~status ); clear;  return; end

% generate and send waveform
[status] = rs_generate_wave( InstrObject, IQInfo, StartARB, KeepLocalFile );
if( ~status ); clear;  return; end

% Apply RF settings if this is not a pure baseband source
if( isempty( strfind( InstrIDN, 'AFQ' ) ) && isempty( strfind( InstrIDN, 'AMU' ) ) )

    disp('Setting up RF...');
        
    % apply some RF settings
    [status, Result] = rs_send_query( InstrObject, 'FREQ 1.0 GHz; *OPC?' );
    if( ~status ); clear;  return; end

    [status, Result] = rs_send_query( InstrObject, 'POW 0.0 dBm; *OPC?' );
    if( ~status ); clear;  return; end

    % switch output ON
    [status, Result] = rs_send_query( InstrObject, 'OUTP:STAT ON; *OPC?' );
    if( ~status ); clear;  return; end

end

% Read the instruments error queue
status = rs_check_instrument_errors( InstrObject );
if( ~status ); clear;  return; end

% delete instrument object
delete( InstrObject );

% clear variables
clear;

return;


